#include "tine4perl.h"

//Variable Declaration
double blob[MAXIMUM_SIZE];
short *sval = (short *)blob;
UINT32 *lval = (UINT32 *)blob;
float *fval = (float *)blob;
double *dval = blob;
char *cval = (char *)blob;
NAME16 *n16val = (NAME16 *)blob;
NAME32 *n32val = (NAME32 *)blob;
NAME48 *n48val = (NAME48 *)blob;
NAME64 *n64val = (NAME64 *)blob;

/*
 * TINE parsing function (used for pyput)
 */
BYTE *parseInputDataString(char *str,int *fmt,int *siz)
{
    BYTE *data=NULL;
    char *s, *c, *tok = " ,;\t\n";
    int i, n, len, base = 10;
    if (str == NULL || fmt == NULL || siz == NULL) return data;
    if (str[0] == '\"')
    {
      if ((c=strchr(&str[1],'\"')) == NULL) return data; /* badly formed */
      *c = 0;
      data = (BYTE *)&str[1];
      *siz = strlen((char *)data);
      *fmt = CF_TEXT;
    }
    else
    {
      len = strlen(str); n = 0;
      data = (BYTE *)calloc(len,64); /* worst case ? */
      for (s=strtok(str,tok); s != NULL; s = strtok(NULL,tok))
      {
        if (*fmt == CF_NULL)
        {
          len = strlen(s);
          for (i=0; i<len; i++) if (!isdigit(s[i])) break;
          if (i == len) /* all digits ! */
          {
            *fmt = CF_LONG;
            base = 10;
          }
          else if (strncmp(s,"0x",2) == 0 || strncmp(s,"0X",2) == 0)
          {
            *fmt = CF_LONG;
            base = 16;
          }
          else if (s[i] == '.')
          {
            *fmt = CF_DOUBLE;
          }
          else
          {
            if (len < 16) *fmt = CF_NAME16;
            else if (len < 32) *fmt = CF_NAME32;
            else if (len < 64) *fmt = CF_NAME64;
            else *fmt = CF_TEXT;
          }
        }
        switch (*fmt)
        {
          case CF_INT16:
            ((UINT16 *)data)[n] = (UINT16)strtol(s,NULL,base); break;
          case CF_INT32:
            ((UINT32 *)data)[n] = strtol(s,NULL,base); break;
          case CF_FLOAT:
            ((float *)data)[n] = (float)strtod(s,NULL); break;
          case CF_DOUBLE:
            ((double *)data)[n] = strtod(s,NULL); break;
          case CF_NAME16:
            strncpy(((NAME16 *)data)[n].name,s,16); break;
          case CF_NAME32:
            strncpy(((NAME32 *)data)[n].name,s,32); break;
          case CF_NAME64:
            strncpy(((NAME64 *)data)[n].name,s,64); break;
          case CF_TEXT:
            strncpy((char *)data,s,len);
            *siz = len;
            return data;
          default:
            break;
        }
        n++;
        if (*siz > 0 && n == *siz) break;
      }
      *siz = n;
    }
    return data;
}




/*
 * Executes the tget TINE command-line command
 */
int tget(char * address, char * property, int size, int format, void * buf, int timeout) 
{
  int status = 0;
  DTYPE dout;
  // Build the TINE message
  memset(&dout,0,sizeof(DTYPE));
  dout.dFormat = format;
  dout.dArrayLength = size;
  dout.data.vptr = buf;
  status = ExecLinkEx(address,property,&dout,NULL,CA_READ|CA_RETRY,timeout);
  return status;
}

/*
 * Executes the tput TINE command-line command
 */
int tput(char * address, char * property, int size,int format, void *data, int timeout) 
{
  int status = 0;
  DTYPE din;

  // Build the TINE message
  memset(&din,0,sizeof(DTYPE));
  din.dFormat = format;
  din.dArrayLength = size;
  din.data.vptr = data;

  status = ExecLinkEx(address,property,NULL,&din,CA_WRITE|CA_RETRY,timeout);

  return status;

}

/*
 * Executes the tput TINE command-line command
 */
int putData(char * address, char * property, char * format, int size, char *datastr, int timeout) 
{
  
  //Convert the format
  int fmt = ftoi((char *) format);  

  //Parse the input data 
  BYTE *data = NULL;
  data = parseInputDataString(datastr,&fmt,&size);

  //Send the data to TINE
  int status = tput(address, property, size, fmt, data,timeout);

  return status;
}


/*
 * 
 */
char ** getData(char* address,char* property,char* format, int size, int timeout)
{
  //Arguments
  char errstr[256];
  char s[65];
  int i = 0;
  int nlen = 0;
  int fmt = 0;
  int sts = 0;
  //char *data[MAXIMUM_SIZE];
  char **data;
	
  //Check the size
  if (size > MAXIMUM_SIZE) size = MAXIMUM_SIZE;

  //Extract the format
  fmt = ftoi((char *) format);

  //Call the C code and returns a Python object
  sts = tget(address, property, size, fmt, blob, timeout);

  //No error in the TINE call
  if (sts == 0 )
  {
	  data = malloc(sizeof(char)*size);	  
	  switch(fmt)
	  {
	  	case CF_LONG:
			for (i=0; i<size; i++) 
			{		
			  data[i] = malloc (sizeof(long));
			  sprintf(data[i],"%d",lval[i]);		
			}	  	
		case CF_SHORT:
			for (i=0; i<size; i++) 
			{		
			  data[i] = malloc (sizeof(short));
			  sprintf(data[i],"%d",sval[i]);
			}
	  	case CF_FLOAT:
			for (i=0; i<size; i++) 
			{		
			  data[i] = malloc (sizeof(float));
			  sprintf(data[i],"%f",fval[i]);
			}
			break;
	  	case CF_DOUBLE:
			for (i=0; i<size; i++) 
			{		
			  data[i] = malloc (sizeof(double));
			  sprintf(data[i],"%e",dval[i]);
			}			
			break;
	  	case CF_TEXT:
			for (i=0; i<size; i++) 
			{		
			  data[i] = malloc (sizeof(char));
			  sprintf(data[i],"%c",cval[i]);
			}			
			break;
	  	case CF_NAME16:
	  	case CF_NAME32:
	  	case CF_NAME48:
	  	case CF_NAME64:			
			nlen = fmtsizeof(fmt);
	  		for (i=0; i<size; i++)
	  		{
		 	  data[i] = malloc (sizeof(char)*nlen);			  	  
	  		  strncpy(s,cval + i*nlen,nlen);
			  s[64] = 0;
	  		  strcpy(data[i],s);
	  		}			
	  		break;  		
	  	default:
			data = malloc(sizeof(char)*2);	
	  		data[0] = malloc (sizeof(char)*64);
			strcpy(data[0],"Data Type not Supported");
			data[1] = 0;
			return &data[0];
			//return "4tine: Error in the TINE call\n";
	  		break;
	  }
	  data[i] = 0;
 	  return &data[0];
	  //Free the memory
	  free(data);	
  }
  else
  {
	  printf("TINE Error: %d\n",sts);
	  data = malloc(sizeof(char)*2);	
	  data[0] = malloc (sizeof(short));
	  sprintf(data[0],"%d",sts);
	  data[1] = 0;
  	  return &data[0];
	  free(data);	
  }
}

